<?php

namespace Drupal\entity_information\Plugin\EntityInformation;

use Drupal\Core\Annotation\Translation;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Menu\MenuLinkManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\entity_information\Plugin\EntityInformationBase;
use Drupal\entity_information\Plugin\EntityInformationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Builds menu link content information.
 *
 * @EntityInformation(
 *   id = "entity_information_menu_link_content",
 *   label = @Translation("Entity Information: Menu link"),
 *   bundles = {
 *     "node.*",
 *   },
 *   description = @Translation("Overview of available menu link(s) for nodes."),
 *   weight = 15,
 *   open = TRUE
 * )
 */
class EntityInformationMenu extends EntityInformationBase implements EntityInformationInterface, ContainerFactoryPluginInterface {

  /**
   * The menu link manager service.
   *
   * @var \Drupal\Core\Menu\MenuLinkManagerInterface
   */
  protected $menuLinkManager;

  /**
   * Constructs a LocalActionDefault object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Menu\MenuLinkManagerInterface $menu_link_manager
   *   The menu link manager service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MenuLinkManagerInterface $menu_link_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->menuLinkManager = $menu_link_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.menu.link')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function getTitle() {
    return $this->t('Menu links');
  }

  /**
   * {@inheritdoc}
   */
  public function view(ContentEntityInterface $entity): array {
    // We can only show menu links when the menu_link_content module is enabled.
    if (!$this->moduleHandler()->moduleExists('menu_link_content')) {
      $this->setNoResultsMessage($this->t('The Custom Menu Links module needs to be enabled to show the menu link information.'));
      return $this->noResultsResponse();
    }

    // Get all menu link content items for this entity.
    $menu_link_content_items = $this->menuLinkManager->loadLinksByRoute('entity.node.canonical', ['node' => $entity->id()]);

    // Get the bundle for the entity.
    $bundle = $this->getBundle($entity);

    // Show a message when no menu link content is found.
    if (empty($menu_link_content_items)) {
      $this->setNoResultsMessage($this->t('No menu links found for this :bundle.',  [':bundle' => $bundle->label()]));
      return $this->noResultsResponse();
    }

    // Build the table header.
    $header = [];
    $header[] = [
      'data' => $this->t('Title'),
      'field' => 'title',
    ];
    $header[] = [
      'data' => $this->t('Menu'),
      'field' => 'menu',
    ];
    $header[] = [
      'data' => $this->t('Language'),
      'field' => 'language_name',
    ];
    $header[] = [
      'data' => $this->t('Operations'),
    ];

    // Build the table rows.
    $destination = $this->getDestinationArray();
    $rows = [];
    /** @var \Drupal\menu_link_content\Plugin\Menu\MenuLinkContent[] $menu_link_content_items */
    foreach ($menu_link_content_items as $menu_link_item) {
      $row = [];

      // Get the menu link content entity.
      $menu_link_content_by_uuid = $this->entityTypeManager()->getStorage('menu_link_content')->loadByProperties([
        'uuid' => $menu_link_item->getDerivativeId(),
      ]);
      /** @var \Drupal\menu_link_content\Entity\MenuLinkContent $menu_link_content */
      $menu_link_content = reset($menu_link_content_by_uuid);

      // Get the menu entity.
      /** @var \Drupal\system\Entity\Menu $menu */
      $menu = $this->entityTypeManager()->getStorage('menu')->load($menu_link_content->getMenuName());

      // Compile the table row and add some operations.
      $row['data']['title'] = $menu_link_content->getTitle();
      $row['data']['menu'] = $menu->label();
      $row['data']['language_name'] = $menu_link_content->language() ? $menu_link_content->language()->getName() : 'Und';

      $operations = [];
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'url' => Url::fromRoute('entity.menu_link_content.edit_form', ['menu_link_content' => $menu_link_content->id()], ['query' => $destination]),
      ];
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'url' => Url::fromRoute('entity.menu_link_content.delete_form', ['menu_link_content' => $menu_link_content->id()], ['query' => $destination]),
      ];
      $row['data']['operations'] = [
        'data' => [
          '#type' => 'operations',
          '#links' => $operations,
        ],
      ];

      // Add the row to the rows array.
      $rows[] = $row;
    }

    // Setup and return the build array for this entity information container.
    return [
      'menu_link_content' => [
        'label' => [
          '#type' => 'item',
          '#title' => $this->t('The following menu links are available for this :bundle:', [':bundle' => $bundle->label()]),
        ],
        'overview' => [
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $rows,
        ],
      ],
    ];
  }

}
