<?php

namespace Drupal\entity_information\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Routing\RedirectDestinationTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\entity_information\EntityInformationHelperTrait;
use Drupal\media\MediaInterface;
use Drupal\node\NodeInterface;

/**
 * Base class for Entity information plugins.
 */
abstract class EntityInformationBase extends PluginBase {

  use EntityInformationHelperTrait;
  use StringTranslationTrait;
  use RedirectDestinationTrait;

  /**
   * The information block title.
   *
   * @var string
   */
  protected $title;

  /**
   * An option to allow or deny access to the information block.
   *
   * @var bool
   */
  protected $access = TRUE;

  /**
   * A message explaining why no results are found.
   *
   * @var string
   */
  protected $noResultsMessage = '';

  /**
   * Gets the title for the information block.
   *
   * @return string
   *   The title for the information block.
   */
  public function getTitle() {
    if (empty($this->title)) {
      return $this->t('Entity Information');
    }
    return $this->title;
  }

  /**
   * Sets the title for the information block.
   *
   * @param string $title
   *   The title for the information block.
   */
  public function setTitle($title) {
    $this->title = $title;
  }

  /**
   * Gets whether access is granted or not.
   *
   * @return bool
   *   The access property value.
   */
  public function getAccess() {
    return $this->access;
  }

  /**
   * Sets the access property value.
   *
   * @param bool $access
   *   The access property value.
   */
  public function setAccess(bool $access) {
    $this->access = $access;
  }

  /**
   * Get the bundle entity for a given entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The current entity.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   A bundle entity.
   */
  public function getBundle(ContentEntityInterface $entity) {
    // @TODO - add other entity types.
    switch ($entity->getEntityTypeId()) {
      case 'media':
        $bundle = $this->entityTypeManager()->getStorage('media_type')->load($entity->bundle());
        break;

      default:
        $bundle = $entity->get('type')->entity;
        break;
    }

    return $bundle;
  }

  /**
   * Gets the message explaining why no results are found.
   *
   * @return string
   *   The message explaining why no results are found.
   */
  public function getNoResultsMessage(): string {
    return $this->noResultsMessage;
  }

  /**
   * Sets the message explaining why no results are found.
   *
   * @param string $no_results_message
   *   The message explaining why no results are found.
   */
  public function setNoResultsMessage(string $no_results_message): void {
    $this->noResultsMessage = $no_results_message;
  }

  /**
   * Returns the message when no results are found.
   *
   * @return array
   */
  public function noResultsResponse(): array {
    return [
      '#markup' => $this->getNoResultsMessage(),
    ];
  }

}
